from log import log
from rule import Rule
from symbols import Symbol, Symbols


class Edge(Rule):
    counter = 0

    def __str__(self):
        span = '{}–{} '.format(self.begin, self.end)
        return span + Rule.__str__(self)

    def __repr__(self):
        return 'Edge(' + str(self) + ')'

    def __init__(self, predicted_from=None, predicted_by_rule=None,
                 extended_from=None, extended_using=None, tokens=None):
        Edge.counter += 1
        self.identificator = Edge.counter

        self.predicted_from = predicted_from
        self.extended_from = extended_from
        if extended_from is not None:
            self._extend_from(extended_from, extended_using, tokens)
        elif predicted_by_rule is not None:
            self.rule = predicted_by_rule
            self.left = Symbol(predicted_by_rule.left)
            Symbols.__init__(self, predicted_by_rule, position=0)
            self.real_offset = 0
            if predicted_from is not None:
                self.begin = predicted_from.end
                self.end = predicted_from.end
                self._predict_attributes()
            else:
                self.begin = 0
                self.end = 0
        if extended_from is None:
            self.children = [None] * len(self)
        self.complete = self.position == len(self)
        # TODO: není spíš než next current?
        self.next_symbol = None if self.complete else self[self.position]

    def _extend_from(self, extended_from, extended_using, tokens):
        self.predicted_from = extended_from.predicted_from
        self.rule = extended_from.rule
        self.begin = extended_from.begin
        self.left = Symbol(extended_from.left)
        Symbols.__init__(self, extended_from,
                         position=extended_from.position + 1)
        # když jde o ten samý seznam, vytvořit nový, protože jde taky o stejnou
        # původní hranu
        if (extended_using is not None and
                extended_from.children is extended_using.children):
            self.children = [None] * len(self)
        else:
            self.children = extended_from.children
        if extended_using is not None:
            self.end = extended_using.end
            self.real_offset = (extended_from.real_offset +
                                extended_using.end - extended_using.begin)
            self.children[extended_from.position] = extended_using
        elif tokens:  # terminal rule
            self.end = extended_from.end + 1
            self.real_offset = extended_from.real_offset + 1
            self[extended_from.position].update(
                tokens[extended_from.begin + extended_from.real_offset])

    def copy_attributes(self, update=None):
        copy_from = self.position - 1
        if update:
            current = self[copy_from]
            for attr, value in sorted(update.items()):
                if value is None:
                    log.debug('null value   %s', attr)
                    continue  # asi by se nemělo kopírovat; nejspíš nemělo
                elif attr not in current or current[attr] is None:
                    log.debug('move         %s=%s → %s', attr, value, current)
                    current[attr] = value
                elif current[attr] != value:
                    log.warning('update       %s=%s → %s', attr, value,
                                current)
                    current[attr] = value

        for attr, indexes in sorted(self.rule.attr_matches.items()):
            if copy_from not in indexes:
                continue  # odsud se ani nemá kopírovat
            elif attr not in self[copy_from]:
                log.warning('no attr      %s %s', attr, self[copy_from])
                continue  # není co kopírovat

            value = self[copy_from][attr]
            if value is None:
                log.debug('null value   %s', attr)
                continue  # asi by se nemělo kopírovat; nejspíš nemělo

            if indexes[0] == -1:  # kopírování do hlavy
                log.debug('copy         %s=%s → head', attr, value)
                self.left[attr] = value

            # kopíruje se jen na členy vpravo od právě vloženého
            for index in indexes[indexes.index(copy_from)+1:]:
                log.debug('copy         %s=%s → %s', attr, value, self[index])
                self[index][attr] = value

    def _predict_attributes(self):
        symbol = self.predicted_from.next_symbol
        for attr, value in self.rule.left.items():
            if value is None and attr in symbol:
                self.left[attr] = symbol[attr]
                # log.debug('copy         %s=%s → head', attr, symbol[attr])

        for attr, match_on_indexes in sorted(self.rule.attr_matches.items()):
            if -1 in match_on_indexes and attr in self.left:
                value = self.left[attr]
                for index in match_on_indexes[1:]:
                    self[index][attr] = value
                    # log.debug('copy         head → %s=%s', attr, value)
