from rule import Rule, permute_rules, insert_into_rules
from rules import Rules

# věty

# TODO: někdy asi přejmenovat S na CLAUSE
# TODO: a taky rozlišovat mezi „preterminálními“ NP a takovými, které už třeba
#       obsahují ukazovací zájmeno, aby se jich nedalo vložit víc (restriktivní
#       gramatika)
# TODO: místo verbals udělat “infinitivals” (?) / infinite

modals = [
    #
    # modální slovesa
    #

    # Chce/musí/lze to odevzdat zítra.
    '(MODAL e g n a m) → (lemma="chtít|muset|lze" e g n a m)',
]

# konstituenty ve volném pořadí
# TODO: kvůli gramatickému pořadí „bude muset“ asi potřebuju ještě filtr
skeletons = Rules(permute_rules((
    '(S e g n a m) → (k=5 e g n a m)',

    '(S e g n a m) → (k=5 e g n a m) (NP)',

    # Učím Petra angličtinu (akuzativ životný a neživotný)
    '(S e g n a m) → (k=5 e g n a m) (NP) (NP)',

    # Petr půjčil Pavlovi knihu
    '(S e g n a m) → (k=5 e g n a m) (NP) (NP) (NP)',

    '(S e g n a m) → (k=5 e g n a m) (ADVP)',

    # Kočka se dychtivě napila
    '(S e g n a m) → (k=5 e g n a m) (NP) (ADVP)',

    # Chce/musí to odevzdat (zítra)
    # TODO: dá se udělat m="[^F]" a zároveň kopírovat modus do S? nejspíš jo
    '(S e g n a) → (MODAL m="[^F]") (k=5 e g n a m=F)',
    '(S e g n a) → (MODAL m="[^F]") (k=5 e g n a m=F) (NP)',
    '(S e g n a) → (MODAL m="[^F]") (k=5 e g n a m=F) (ADVP)',
    '(S e g n a) → (MODAL m="[^F]") (k=5 e g n a m=F) (NP) (ADVP)',
)))

rules = skeletons + [
    # Bude to odevzdávat až za měsíc.
    '(S e g n a m=B) → (k=5 lemma="být" e g n a m=B) (VP m=F a=I)',

    # Bude to muset odevzdat/odevzdávat až za měsíc.
    '(S e g n a m=B) → (k=5 lemma="být" e g n a m=B) (VP e g n a m=F)',

    # Umožňuje nám stanovit příznivější ceny pro domácí klienty.
    '(S a e g m n) → (k=5 a e g m n) (VP m=F a=P)',

    # Chtěli [jsme tam] jít, ale nakonec [jsme si to] rozmysleli.

    # Chtěl jsem [lyžovat nebo bruslit].
    # (INF) → (INF) (k=8) (INF)
    # (INF) → (k=5 m=F)
    # (INF) → (k=5) (CLITICS) (NP)
    # (S) → (MODAL) (CLITICS) (INF)
    # (NP) → (k=3 x=D) (NP)
    # (NP) → (k=1)
]

rules = [Rule(rule).insert_symbol('(CLITICS)', position=1) for rule in rules]
rules = [rule.insert_symbol('(INTER)', before=True, after=True)
         for rule in rules]
rules = [rule + '(OPT_FULLSTOP)' for rule in rules]

# TODO: tvořit kombinace pomocí iterátoru, který symbol vloží postupně na
# všechny pozice
kombinace = '''
Chtěli obědvat
Chtěli jsme obědvat
Naši chtěli obědvat
V sobotu chtěli obědvat
Chtěli obědvat až po nás
Naši nechtěli ani obědvat

musím jít domů
domů prý jít musím
'''

rules += modals

rules += insert_into_rules((
    #
    # přívlastkové vedlejší věty
    #

    # kterého bodla včela („který se smál“ by mohl být druhý případ – se shodou,
    # ale zatím to asi není potřeba (interpretace se z toho dostane i tak)
    '(ATTR_CLAUSE g n) → (lemma="který" g n) (CLITICS) (S)',

    # TODO: a co pád?
    '(ATTR_CLAUSE g n) → (word=na) (lemma="který" g n) (S)',

    '(ATTR_CLAUSE) → (word=co) (S m="[^f]")',

    #
    # TODO: předmětové vedlejší věty
    #
    # vypadá to, že to není úplně jednoduchý
    #   *že utekli jsme z hodiny
    #   že jsme utekli z hodiny (iniciální, postiniciální)
    #   že ten kluk se ulil z hodiny (preiniciální, iniciální, postiniciální)
    #   že se ulil z hodiny (iniciální, postiniciální)
    #

    # že/co jsem (to) nevěděl
    # že maminka se (asi) ztratila
    '(OBJ_CLAUSE) → (OPT_COMMA) (word="že|co") (S) (OPT_COMMA)',

    # kdo to nevěděl
    # komu jsem to nedal
    # na koho naléhal
), '(OPT_COMMA)', before=True, inside=False, after=True)


# zapracovat spony a tak
rules += [
    # '(S) → (NP) (CLITICS) (VP a e g m n)',
    # '(S) → (S) (k=8) (S)',
    # '(S) → (S) (OPT_COMMA) (S)',

    # tranzitní skelet (MEDIAL?)
    # pomocné finitní sloveso, modální sloveso, plnovýznamové sloveso
    '(TRANSITIIONAL) → (TR_PR) (TR_PR_PLUS) (TR)',
    # Petr si to | bude muset přečíst | znovu

    # Bude to odevzdávat až za měsíc.
    # Petr bude dělat zkoušku z chemie.
    '(VP e g n a m=B) → (k=5 lemma="být" e g n a m=B) (k=5 m=F) (NP)',
    # TODO: To je strašný, opravdu budu muset udělat nějakej generátor, kterej
    # zaplní tranzitní skelet. Např.:
    # '(VP) → (bude) ten úkol (muset) nejspíš (odevzdat) až příští měsíc'
    # '(VP OBJ_ACC OBJ_DAT) → [Petr] ten úkol (bude) nejspíš (muset) hodně
    # rychle (odevzdat) tomu učiteli'

    # – ten generátor bude muset upravovat obě strany pravidla
    # – přidá třeba OBJ_ACC a OBJ_ACC2/EXTRA (víc jich nečekám):
    #   (VP OBJ_ACC OBJ_ACC2) → (k=5 m="[^F]") (OBJ_ACC)        (OBJ_ACC2)
    #                               učím     Petra (+ANIM)   angličtinu (−ANIM)
    # – životnost bohužel u ženskýho rodu nevím, to je pěkně blbý
    #   musela by se brát z WordNetu, majka to prostě mít nebude

    # Bude to muset odevzdat/odevzdávat až za měsíc.
    '(VP m=F) → (MODAL m=F) (k=5 m=F)',
    '(VP a m=F) → (MODAL m=F) (k=5 a m=F) (ADVP)',

    # '(COPULA e g n a m) → (lemma="být" e g n a m)',

    # (všichni) jsme hladoví
    '(VP e g n a m) → (lemma="být" e g n a m) (ADJP)',

    # TODO: má smysl zpracovávat spony odděleně od ostatních sloves, protože
    # ostatní nebudou mít „Petr je učitel“ (nominativ)

    # On je pytel blech.
    # '(VP e g n a m) → (lemma="být" e g n a m) (NP)',  # je [něco]
]
